# Advanced Logging Module
import logging

# logging.DISABLE = 100
import pytz
import datetime
import traceback

# Reset
RESET = '\033[0m'

# Regular Colors
BLACK = '\033[30m'
RED = '\033[31m'
GREEN = '\033[32m'
YELLOW = '\033[33m'
BLUE = '\033[34m'
MAGENTA = '\033[35m'
CYAN = '\033[36m'
WHITE = '\033[37m'

# Bold
BOLD_BLACK = '\033[1;30m'
BOLD_RED = '\033[1;31m'
BOLD_GREEN = '\033[1;32m'
BOLD_YELLOW = '\033[1;33m'
BOLD_BLUE = '\033[1;34m'
BOLD_MAGENTA = '\033[1;35m'
BOLD_CYAN = '\033[1;36m'
BOLD_WHITE = '\033[1;37m'

# Underline
UNDERLINE_BLACK = '\033[4;30m'
UNDERLINE_RED = '\033[4;31m'
UNDERLINE_GREEN = '\033[4;32m'
UNDERLINE_YELLOW = '\033[4;33m'
UNDERLINE_BLUE = '\033[4;34m'
UNDERLINE_MAGENTA = '\033[4;35m'
UNDERLINE_CYAN = '\033[4;36m'
UNDERLINE_WHITE = '\033[4;37m'

# High Intensity
INTENSITY_BLACK = '\033[0;90m'
INTENSITY_RED = '\033[0;91m'
INTENSITY_GREEN = '\033[0;92m'
INTENSITY_YELLOW = '\033[0;93m'
INTENSITY_BLUE = '\033[0;94m'
INTENSITY_MAGENTA = '\033[0;95m'
INTENSITY_CYAN = '\033[0;96m'
INTENSITY_WHITE = '\033[0;97m'


# List of timezone: https://gist.github.com/heyalexej/8bf688fd67d7199be4a1682b3eec7568
class ColoredLogHandler(logging.StreamHandler):
    def __init__(self, enable_levelname=True, enable_asctime=False, enable_thread_name=False, enable_traceback=False, timezone='Asia/Seoul'):
        super().__init__()
        self.setLevel(logging.DEBUG)
        self.setFormatter(self.__LogFormatter(enable_levelname=enable_levelname,
                                              enable_asctime=enable_asctime,
                                              enable_thread_name=enable_thread_name,
                                              enable_traceback=enable_traceback,
                                              timezone=timezone))

    class __LogFormatter(logging.Formatter):
        def __init__(self, enable_levelname, enable_asctime, enable_thread_name, enable_traceback, timezone):
            self.enable_levelname = enable_levelname
            self.enable_asctime = enable_asctime
            self.enable_thread_name = enable_thread_name
            self.enable_traceback = enable_traceback
            self.timezone = timezone
            self.loglevel_length = {
                logging.DEBUG   : 5,
                logging.INFO    : 4,
                logging.WARNING : 7,
                logging.ERROR   : 5,
                logging.CRITICAL: 8
            }

            def ___FORMATSPACEBACK___(format_name: str, pad_length: int):
                return f'[%({format_name})s]{" " * pad_length}: '

            def ___LEVELNAME___(loglevel):
                return ___FORMATSPACEBACK___("levelname", 8 - self.loglevel_length[loglevel]) if self.enable_levelname else ""

            ___ASCTIME___ = f'[%(asctime)s] ' if self.enable_asctime else ""
            ___THREADNAME___ = f'(%(threadName)s)' if self.enable_thread_name else ""

            __FORMAT_DEBUG = f'{___ASCTIME___}{___LEVELNAME___(logging.DEBUG)}%(message)s <%(filename)s:%(funcName)s{___THREADNAME___}:%(lineno)d>'
            __FORMAT_INFO = f'{___ASCTIME___}{___LEVELNAME___(logging.INFO)}%(message)s'
            __FORMAT_WARNING = f'{___ASCTIME___}{___LEVELNAME___(logging.WARNING)}%(message)s'
            __FORMAT_ERROR = f'{___ASCTIME___}{___LEVELNAME___(logging.ERROR)}%(message)s <%(filename)s:%(funcName)s{___THREADNAME___}:%(lineno)d>'
            if self.enable_traceback:
                __FORMAT_CRITICAL = f'{___ASCTIME___}{___LEVELNAME___(logging.CRITICAL)}<%(pathname)s:%(funcName)s{___THREADNAME___}:%(lineno)d>\n%(message)s'
            else:
                __FORMAT_CRITICAL = f'{___ASCTIME___}{___LEVELNAME___(logging.CRITICAL)}%(message)s <%(pathname)s:%(funcName)s{___THREADNAME___}:%(lineno)d>'

            self.FORMATS = {
                logging.DEBUG   : GREEN + __FORMAT_DEBUG + RESET,
                logging.INFO    : CYAN + __FORMAT_INFO + RESET,
                logging.WARNING : YELLOW + __FORMAT_WARNING + RESET,
                logging.ERROR   : MAGENTA + __FORMAT_ERROR + RESET,
                logging.CRITICAL: RED + __FORMAT_CRITICAL + RESET
            }

        def format(self, record):
            log_fmt = self.FORMATS.get(record.levelno)
            formatter = logging.Formatter(log_fmt)
            # @@@ Print traceback when loglevel is CRITICAL
            if self.enable_traceback and record.levelno == logging.CRITICAL:
                record.msg += "\n" + "".join(traceback.format_stack())
            # @@@ Set timezone
            # [2023-12-17 19:28:37,296]
            formatter.formatTime = self.formatTime
            formatter.datefmt = '%Y-%m-%d %H:%M:%S.%f'

            # record.levelname = record.levelname.center(self.levelname_pad)    # Centerpad is not pretty
            return formatter.format(record)

        def formatTime(self, record, datefmt=None):
            dt = datetime.datetime.fromtimestamp(record.created, tz=pytz.UTC).astimezone(pytz.timezone(self.timezone))
            if datefmt:
                s = dt.strftime(datefmt)
            else:
                try:
                    s = dt.isoformat(timespec='milliseconds')
                except TypeError:
                    s = dt.isoformat()
            return s


logging.basicConfig(level=logging.DEBUG, handlers=[ColoredLogHandler(
        enable_levelname=True,
        enable_asctime=True,
        enable_thread_name=False,
        enable_traceback=False)])

if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG, handlers=[ColoredLogHandler(
            enable_levelname=True,
            enable_asctime=True,
            enable_thread_name=False,
            enable_traceback=False)])

    logging.debug('This message is a log message.')
    logging.info('This message is a log message.')
    logging.warning('This message is a log message.')
    logging.error('This message is a log message.')
    logging.critical('This message is a log message.')



from zeroset import cv0
import cv2

files = cv0.glob("")

for file in files:
    img = cv0.imread(file)
    img = cv2.rotate(img, cv2.ROTATE_90_COUNTERCLOCKWISE)
    cv0.imwrite(file, img)
    # cv0.imshow(img).waitKey()



from tabulate import tabulate
import sys
import cv2
from PIL import Image, ImageDraw, ImageFont
import numpy as np


def from_pil(img_pil):
    return cv2.cvtColor(np.array(img_pil), cv2.COLOR_RGB2BGR)


def center_pad(img: np.ndarray, width: int, height: int, value=33):
    """
    Places an image at the size you specify, centered and keep ratio.
    :param img: opencv image
    :param width: width
    :param height: height
    :param value: pad value(int or tuple)
    :return: opencv image
    """
    channel = 1 if len(img.shape) == 2 else img.shape[2]
    if isinstance(value, int):
        value = tuple([value] * channel)
    dst = np.zeros((height, width, channel), dtype=np.uint8) + np.array(value, dtype=np.uint8)
    dx = (dst.shape[1] - img.shape[1]) // 2
    dy = (dst.shape[0] - img.shape[0]) // 2
    dst[dy:dy + img.shape[0], dx:dx + img.shape[1]] = img
    return dst


def create_text_image(text, font_path, font_size=20, line_spacing=5, color='black', title=None):
    # 모노스페이스 폰트 로드
    font = ImageFont.truetype(font_path, font_size)

    # 텍스트를 줄바꿈하여 리스트로 변환
    lines = text.split('\n')

    if title is not None:
        lines = [title, ""] + lines

    # 이미지 크기 추정
    max_line_width = max([font.getsize(line)[0] for line in lines])
    total_height = (font_size + line_spacing) * len(lines)

    # 흰색 배경 이미지 생성
    image = Image.new('RGB', (max_line_width, total_height), color='white')
    draw = ImageDraw.Draw(image)

    # 각 줄의 텍스트를 이미지에 추가
    y_text = 0
    for idx, line in enumerate(lines):
        if title is not None and idx == 0:
            draw.text((0, y_text), line, font=font, fill="blue")
        else:
            draw.text((0, y_text), line, font=font, fill=color)
        y_text += font_size + line_spacing

    return image


table = [["Sun", 696000, 1989100000], ["Earth", 6371, 5973.6], ["Moon", 1737, 73.5], ["Mars", 3390, 641.85]]
headers = ["Planet", "R (km)", "mass (x 10^29 kg)"]

table_formats = ["plain", "simple", "github", "grid", "simple_grid",
                 "rounded_grid", "heavy_grid", "mixed_grid", "double_grid",
                 "fancy_grid", "outline", "simple_outline", "rounded_outline",
                 "heavy_outline", "mixed_outline", "double_outline", "fancy_outline",
                 "pipe", "orgtbl", "asciidoc", "jira", "presto", "pretty", "psql",
                 "rst", "youtrack",
                 "latex", "latex_raw", "latex_booktabs", "latex_longtable", "textile", "tsv"]
["moinmoin", "html", "unsafehtml", "mediawiki", ]
# table = []
# for i in range(0, len(table_formats), 3):
#     row = table_formats[i:i + 3]
#     table.append(row)
#
# print(tabulate(table, headers=["-","-","-","-","-"],tablefmt="github"))

imgs = []
for table_format in table_formats:
    print(f'## {table_format}')

    text = tabulate(table, headers=headers, tablefmt=table_format)
    print(text)
    img = create_text_image(text, "CascadiaMono.ttf", title=f'table_format: {table_format}')
    img = from_pil(img)
    img = center_pad(img, img.shape[1] + 50, img.shape[0] + 50, 255)
    imgs.append(img)

MAX_WIDTH = max(img.shape[1] for img in imgs)
palettes = []
for img in imgs:
    palette = np.zeros((img.shape[0], MAX_WIDTH, 3), dtype=np.uint8)
    palette.fill(255)
    palette[0:img.shape[0], 0:img.shape[1]] = img.copy()
    palettes.append(palette.copy())
    # cv2.imshow("palette", palette)
    # cv2.waitKey(0)

sss = cv2.vconcat(palettes)
cv2.imwrite("tablefmt.png", sss)



# py basic advanced util functions
import os, sys
import inspect
import numpy as np
from tabulate import tabulate
import urllib.parse
import math
import pickle
import json
from typing import *


def print_table(*args, tablefmt="simple_grid", no_print=False) -> str:
    """
    print table format
    :param args: same as python's print args
    :param tablefmt: tabulate's tablefmt
    :param no_print: not print if True
    :return: printed string
    """

    def parse_dict(name, d):
        return [[f'{name}["{k}"]', v] for k, v in d.items()]

    variable_names = []  # variable name, value,
    for k, v in inspect.currentframe().f_back.f_locals.items():
        variable_names.append([k, v])
        if isinstance(v, dict):
            variable_names += [e for e in parse_dict(k, v)]
    table = []
    for arg in args:
        # variable_name = [k for k, v in inspect.currentframe().f_back.f_locals.items() if v is arg]
        variable_name = [e[0] for e in variable_names if e[1] is arg]
        variable_name = variable_name[-1] if len(variable_name) > 0 else "CONSTANT"
        # if isinstance(arg, np.ndarray):
        #     print(f'{variable_name} (type={type(arg).__name__},shape={arg.shape})\n{arg}')
        # else:
        #     print(f'{variable_name} (type={type(arg).__name__}): {arg}')
        if isinstance(arg, np.ndarray):
            typename = f'{type(arg).__name__}\n - shape: {arg.shape}'
            val = arg
        elif isinstance(arg, list):
            typename = f'{type(arg).__name__}\n - len: {len(arg)}'
            val = str(arg)
            line_length = 100
            val = "\n".join([val[i:i + line_length] for i in range(0, len(val), line_length)])
        else:
            typename = type(arg).__name__
            val = arg
        table.append([variable_name, typename, val])
    headers = ["Variable", "Type", "Value"]
    print_string = tabulate(table, headers=headers, tablefmt=tablefmt)
    if not no_print:
        print(print_string)
    return print_string


def print_with_name(*args, no_print=False) -> List[str]:
    """
    print name and value
    :param args: same as python's print args
    :param no_print: not print if True
    :return: printed string
    """

    def parse_dict(name, d):
        return [[f'{name}["{k}"]', v] for k, v in d.items()]

    variable_names = []  # variable name, value,
    for k, v in inspect.currentframe().f_back.f_locals.items():
        variable_names.append([k, v])
        if isinstance(v, dict):
            variable_names += [e for e in parse_dict(k, v)]
    printed_strings = []
    for arg in args:
        variable_name = [e[0] for e in variable_names if e[1] is arg]
        variable_name = variable_name[-1] if len(variable_name) > 0 else "CONSTANT"
        print_string = f'{variable_name}: {arg}'
        if not no_print:
            print(print_string)
        printed_strings.append(print_string)
    return printed_strings


def list_chunk(lst: List, n: int):
    return [lst[i:i + n] for i in range(0, len(lst), n)]


def url_decode(text_encoded):
    return urllib.parse.unquote(text_encoded.replace("+", "%20"))


def _format_file_size(size_bytes: int, precision: int = 2, padding: str = ''):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, precision)
    return f'{s}{padding}{size_name[i]}'


def get_value_size(val):
    # 이거 dict 는 제대로 측정을 못하는데 pickle로 dumps 하고 측정해야 할듯
    return _format_file_size(sys.getsizeof(pickle.dumps(val)))


def get_file_size(file):
    return _format_file_size(os.path.getsize(file))


def print_auto(*args):
    for arg in args:
        print(f'-----type-----{type(arg).__name__}')
        if isinstance(arg, list):
            print(arg)
        elif isinstance(arg, dict):
            print(json.dumps(arg, indent=4, ensure_ascii=False))
        elif isinstance(arg, np.ndarray):
            print(arg.tolist())
        else:
            print(arg)


if __name__ == '__main__':
    arr = np.random.random((2, 3))
    d = {
        "a": 1,
        "b": 2,
        "c": 3,
        "x": {
            "y": 777,
            "z": 888
        }
    }
    c = 666
    lst = [1, 2, 3] * 50
    lst2 = [[1, 2], [3, 4], [5, 6]] * 50

    print_function = print_auto

    print_function(arr)
    print_function(c)
    print_function(d["a"])
    print_function(d)
    print_function(d["x"])
    print_function(arr, c, d["b"], d)
    print_function(lst2)



import selenium
from selenium import webdriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.common.by import By
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.support.select import Select
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.common import WebDriverException
from selenium.webdriver.chrome.service import Service
import os
import chromedriver_autoinstaller
import shutil
import screeninfo

CHROME_DRIVER_PATH = chromedriver_autoinstaller.install()


def get_selenium_driver(headless: bool = False, incognito: bool = False, download_dir: str = None) -> WebDriver:
    monitors = screeninfo.get_monitors()
    width, height = (monitors[0].width, monitors[0].height) if len(monitors) != 0 else (1024, 768)

    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('lang=ko')
    chrome_options.add_argument(f'window-size={width},{height}')
    chrome_options.add_argument("user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36")
    if incognito:
        chrome_options.add_argument('--incognito')
    if headless:
        chrome_options.add_argument('--headless=new')

    prefs = {
        "safebrowsing.enabled": True,
    }
    if download_dir is not None:
        download_dir = os.path.abspath(download_dir)
        if not os.path.exists(download_dir):
            os.mkdir(download_dir)
        prefs["download.default_directory"] = download_dir
        prefs["download.prompt_for_download"] = False
        prefs["download.directory_upgrade"] = True
        prefs["profile.default_content_setting_values.automatic_downloads"] = 1
    chrome_options.add_experimental_option("prefs", prefs)

    service = Service(CHROME_DRIVER_PATH)
    driver = webdriver.Chrome(options=chrome_options, service=service)
    return driver


def get_element(element, key, value, timeout=30):
    WebDriverWait(element, timeout).until(EC.presence_of_element_located((key, value)))
    return element.find_element(key, value)


def get_elements(element, key, value, timeout=30):
    WebDriverWait(element, timeout).until(EC.presence_of_element_located((key, value)))
    return element.find_elements(key, value)


def get_alert_text(driver, timeout=30):
    WebDriverWait(driver, timeout).until(EC.alert_is_present())
    r = driver.switch_to.alert
    text = r.text
    r.accept()
    return text


def get_any_element(element, key1, value1, key2, value2, timeout=30):
    def any_of(*expected_conditions):
        def any_of_condition(driver):
            for idx, expected_condition in enumerate(expected_conditions):
                try:
                    result = expected_condition(driver)
                    if result:
                        return idx, result
                except WebDriverException:
                    pass
            return False

        return any_of_condition

    return WebDriverWait(element, timeout).until(any_of(
            EC.presence_of_element_located((key1, value1)),
            EC.presence_of_element_located((key2, value2))))


if __name__ == '__main__':
    driver = get_selenium_driver(headless=False, incognito=True)
    driver.get("")
    print(type(driver).__name__)
    driver.close()
    driver.quit()



import inspect
import os
import sys

real_path = os.path.dirname(os.path.abspath(__file__)).replace("\\", "/")
sys.path.append(real_path)

try:
    import zeroset.cv0 as cv0
    import zeroset.py0 as py0
    import zeroset.log0 as log0
    from zeroset.log0 import ColoredLogHandler
    import logging

    import zeroset.f0 as f0
    import zeroset.viz0 as viz0
except ImportError as e:
    print(e, "Import Error")
    exit(1)

__all__ = [name for name, obj in locals().items()
           if not (name.startswith('_') or inspect.ismodule(obj))]

# cv0: Extended version for cv2
# fmt0 : Extended format convert functions
# l0: colored log
# py0: make python function more easier
# s0 : selenium
# v0 : about visualization



# 2024.09.18
import os
import cv2
import numpy as np
import re
from typing import *
import base64
import io
from PIL import Image, ImageFont, ImageDraw
from dataclasses import dataclass
import urllib.request
import sys
import platform
from image_similarity_measures import quality_metrics
import imagehash


class similarity:
    @staticmethod
    def _quality_metrics(img1: np.ndarray, img2: np.ndarray, func: Callable[[np.ndarray, np.ndarray], float], size: Optional[Union[int, Tuple[int, ...]]] = None):
        """
        Compares two images using a function
        :param img1: opencv image(np.ndarray)
        :param img2: opencv image(np.ndarray)
        :param func: Functions in image_similarity_measures.quality_metrics
        :param size: Automatically resize the image if size is None.
                     If size is int, resize both width and height to the same size.
                     If size is a tuple, resize width,height to the specified value.
        :return: The return of the specified function(float)
        """
        if size is None:
            h1, w1 = img1.shape[:2]
            h2, w2 = img2.shape[:2]
            w, h = min(w1, w2), min(h1, h2)
        elif isinstance(size, int):
            w, h = size, size
        elif isinstance(size, tuple):
            w, h = size[0], size[1]
        else:
            raise TypeError("")
        img1r = cv2.resize(img1, (w, h))
        img2r = cv2.resize(img2, (w, h))
        return func(img1r, img2r)

    @staticmethod
    def rmse(img1: np.ndarray, img2: np.ndarray, size: Optional[Union[int, Tuple[int, ...]]] = None):
        # Root mean square error (RMSE)
        return similarity._quality_metrics(img1, img2, quality_metrics.rmse, size)

    @staticmethod
    def psnr(img1: np.ndarray, img2: np.ndarray, size: Optional[Union[int, Tuple[int, ...]]] = None):
        # Peak signal-to-noise ratio (PSNR)
        return similarity._quality_metrics(img1, img2, quality_metrics.psnr, size)

    @staticmethod
    def ssim(img1: np.ndarray, img2: np.ndarray, size: Optional[Union[int, Tuple[int, ...]]] = None):
        # Structural Similarity Index (SSIM)
        return similarity._quality_metrics(img1, img2, quality_metrics.ssim, size)

    @staticmethod
    def fsim(img1: np.ndarray, img2: np.ndarray, size: Optional[Union[int, Tuple[int, ...]]] = None):
        # Feature-based similarity index (FSIM)
        return similarity._quality_metrics(img1, img2, quality_metrics.fsim, size)

    @staticmethod
    def sre(img1: np.ndarray, img2: np.ndarray, size: Optional[Union[int, Tuple[int, ...]]] = None):
        # Signal to reconstruction error ratio (SRE)
        return similarity._quality_metrics(img1, img2, quality_metrics.sre, size)


class hash:
    @staticmethod
    def _get_image_hash(img1: np.ndarray, func: Callable[[np.ndarray], float]):
        ret = func(to_pil(img1))
        return ret

    @staticmethod
    def average_hash(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.average_hash)

    @staticmethod
    def colorhash(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.colorhash)

    @staticmethod
    def phash(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.phash)

    @staticmethod
    def phash_simple(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.phash_simple)

    @staticmethod
    def whash(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.whash)

    @staticmethod
    def dhash(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.dhash)

    @staticmethod
    def dhash_vertical(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.dhash_vertical)

    @staticmethod
    def crop_resistant_hash(img1: np.ndarray):
        return hash._get_image_hash(img1, imagehash.crop_resistant_hash)


GLOB_SORT_DEFAULT = 0
GLOB_SORT_NAT = 1
GLOB_SORT_SIZE = 2


def glob(pathstr: str, ext: str = None, recursive: bool = False, sort_method: int = GLOB_SORT_DEFAULT):
    import glob
    pathstr = re.sub('([\[\]])', '[\\1]', pathstr)
    if ext is not None:
        if ext.startswith("**"):
            recursive = True
        pathstr = os.path.join(pathstr, ext)
    files = glob.glob(pathstr, recursive=recursive)
    if sort_method == GLOB_SORT_NAT:
        import natsort
        # files = natsort.natsorted(files)
        files = natsort.humansorted(files)
    elif sort_method == GLOB_SORT_SIZE:
        files.sort(key=lambda x: os.path.getsize(x))
    return files


def from_blank(width: int, height: int, BGR: Tuple[int, int, int] = (0, 0, 0)) -> np.ndarray:
    return np.zeros((height, width, 3), dtype=np.uint8) + np.array(BGR, dtype=np.uint8)


########## from ? image ##########

# def from_plt(plt, fig):
#     plt.gcf().canvas.get_renderer()
#     img_pil = Image.frombytes('RGB', fig.canvas.get_width_height(), fig.canvas.tostring_rgb())
#     return cv2.cvtColor(np.array(img_pil), cv2.COLOR_RGB2BGR)

def from_url(url: str):
    try:
        headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'}
        request = urllib.request.Request(url, headers=headers)
        response = urllib.request.urlopen(request)
        image_data = response.read()
        image_array = np.asarray(bytearray(image_data), dtype=np.uint8)
        image = cv2.imdecode(image_array, cv2.IMREAD_COLOR)
        if image is None:  # gif일 경우
            gif = Image.open(io.BytesIO(image_data))
            gif.seek(0)
            image = np.array(gif.convert('RGB'))
        return image
    except Exception as e:
        print(e)
        return None


def from_pil(img_pil):
    return cv2.cvtColor(np.array(img_pil), cv2.COLOR_RGB2BGR)


def from_base64(b64str: str):
    img = Image.open(io.BytesIO(base64.b64decode(b64str)))
    return cv2.cvtColor(np.array(img), cv2.COLOR_BGR2RGB)


def from_bytes(img_bytes):
    return cv2.cvtColor(np.array(Image.open(io.BytesIO(img_bytes))), cv2.COLOR_RGB2BGR)


########## to ? image ##########
def to_base64(img: np.ndarray, ext: str = ".png", params=None):
    return base64.b64encode(cv2.imencode(ext, img, params)[1]).decode("utf-8")


def to_pil(img):
    return Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))


def to_bytes(img: np.ndarray, ext: str = ".png"):
    return cv2.imencode(ext, img)[1].tobytes()


def to_color(img: np.ndarray):
    if len(img.shape) == 2 or img.shape[2] == 1:
        img = cv2.cvtColor(img, cv2.COLOR_GRAY2BGR)
    return img


def imread(filename: str, flags=cv2.IMREAD_COLOR):
    """
    This imread method supports korean path
    :param filename: image file paths
    :param flags: image open mode flags
    :return: np.ndarray image
    """
    return cv2.imdecode(np.fromfile(filename, np.uint8), flags)


def imwrite(filename: str, img: np.ndarray, params=None):
    """
    This imread method supports korean path
    :param filename: image file path
    :param img: np.ndarray image
    :param params: cv2.imwrite flags
    :return: True if success else False
    """
    r, eimg = cv2.imencode(os.path.splitext(filename)[1], img, params)
    if r:
        with open(filename, mode="wb") as f:
            eimg.tofile(f)
    return r


def imreads(filepaths: str, flags=cv2.IMREAD_COLOR) -> List[np.ndarray]:
    """
    :param filepaths: image file paths
    :param flags:image open mode flags
    :return: images
    """
    return [imread(filepath, flags) for filepath in filepaths]


def waitKey(delay=0):
    return cv2.waitKey(delay)


def waitESC(delay=1):
    return cv2.waitKey(delay) == 27


def exit():
    sys.exit()


@dataclass
class _wndwait:
    waitKey = waitKey
    waitESC = waitESC


IMSHOW_DEFAULT = 0
IMSHOW_BEST = 1
IMSHOW_AUTOSIZE = 2
IMSHOW_FULLSCREEN = 3
IMSHOW_CV2 = 4


def imshow(arg1: Any, arg2: Any = None, mode: int = IMSHOW_BEST):
    """
    :param arg1: window name or image
    :param arg2: image if arg1 is window name else ignored
    :param mode: Just use IMSHOW_BEST
    :return: opencv image window utility class
    """
    if not platform.platform().startswith('Windows'):
        mode = IMSHOW_DEFAULT
    if isinstance(arg1, str):
        winname, img = arg1, arg2
    else:
        import inspect
        winnames = [k for k, v in inspect.currentframe().f_back.f_locals.items() if v is arg1]
        winname = winnames[-1] if len(winnames) > 0 else "default"
        img = arg1
    if mode == IMSHOW_BEST:
        import screeninfo
        monitor = [m for m in screeninfo.get_monitors() if m.is_primary == True][0]
        if not cv2.getWindowProperty(winname, cv2.WND_PROP_VISIBLE):
            cv2.namedWindow(winname, cv2.WINDOW_NORMAL | cv2.WINDOW_KEEPRATIO)
            nw, nh = resize(img, width=monitor.width, height=monitor.height, return_size=True)
            cv2.resizeWindow(winname, nw, nh)
            cv2.moveWindow(winname, (monitor.width - nw) // 2, (monitor.height - nh) // 2)
        else:
            _, _, cw, ch = cv2.getWindowImageRect(winname)
            window_ratio = cv2.getWindowProperty(winname, cv2.WND_PROP_ASPECT_RATIO)
            image_ratio = img.shape[1] / img.shape[0]
            if abs(image_ratio - window_ratio) > 0.1:
                nw, nh = resize(img, width=cw, return_size=True)
                cv2.resizeWindow(winname, nw, nh)
    elif mode == IMSHOW_AUTOSIZE:
        cv2.namedWindow(winname, cv2.WINDOW_AUTOSIZE)
    elif mode == IMSHOW_FULLSCREEN:
        cv2.namedWindow(winname, cv2.WINDOW_NORMAL)
        cv2.setWindowProperty(winname, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
        _, _, cw, ch = cv2.getWindowImageRect(winname)
        img = resize(img, width=cw, height=ch, return_size=False)
        img = center_pad(img, cw, ch, 33)
    elif mode == IMSHOW_CV2:
        pass

    cv2.imshow(winname, img)
    return _wndwait


def resize(img: np.ndarray, width=None, height=None, interpolation=cv2.INTER_AREA, return_size=False):
    """
    Given both width and height, choose the smaller of the resulting sizes.
    :param img: opencv image
    :param width: width to change
    :param height: height to change
    :param interpolation: interpolation
    :return: opencv image
    """
    h, w = img.shape[:2]
    dims = []
    if height is not None:
        ratio = height / h
        dims.append((int(w * ratio), height))
    if width is not None:
        ratio = width / w
        dims.append((width, int(h * ratio)))
    if len(dims) == 2 and dims[0] > dims[1]:
        dims = dims[1:]
    if len(dims) == 0:
        return img if not return_size else (w, h)

    return cv2.resize(img, dims[0], interpolation=interpolation) if not return_size else dims[0]


def overlay(bgimg3c: np.ndarray, fgimg4c: np.ndarray, coord=(0, 0), inplace=True):
    """
    Overlay a 4-channel image on a 3-channel image
    :param bgimg3c: background 3c image
    :param fgimg4c: foreground 4c image
    :param coord: Coordinates of the bgimg3c  to overlay
    :param inplace: If true, bgimg3c is changed
    :return: Overlaid image
    """
    # if bgimg3c.shape[:2] != fgimg4c.shape[:2]:
    #     raise ValueError(bgimg3c.shape[:2], fgimg4c.shape[:2])
    h, w = fgimg4c.shape[:2]
    crop = bgimg3c[coord[0]:coord[0] + h, coord[1]:coord[1] + w]
    b, g, r, a = cv2.split(fgimg4c)
    mask = cv2.merge([a, a, a])
    fgimg3c = cv2.merge([b, g, r])
    mask = mask / 255.0
    mask_inv = 1.0 - mask
    ret = (crop * mask_inv + fgimg3c * mask).clip(0, 255).astype(np.uint8)
    if inplace:
        bgimg3c[coord[0]:coord[0] + h, coord[1]:coord[1] + w] = ret
    return ret


def center_pad(img: np.ndarray, width: int, height: int, value: Any = 33):
    """
    Places an image at the size you specify, centered and keep ratio.
    :param img: opencv image
    :param width: width
    :param height: height
    :param value: pad value(int or tuple)
    :return: opencv image
    """
    channel = 1 if len(img.shape) == 2 else img.shape[2]
    if isinstance(value, int):
        value = tuple([value] * channel)
    dst = np.zeros((height, width, channel), dtype=np.uint8) + np.array(value, dtype=np.uint8)
    dx = (dst.shape[1] - img.shape[1]) // 2
    dy = (dst.shape[0] - img.shape[0]) // 2
    dst[dy:dy + img.shape[0], dx:dx + img.shape[1]] = img
    return dst


def letterbox(img: np.ndarray, value: Any):
    """
    Put a pad value on the image to change it to a 1:1 aspect ratio.
    :param img: opencv image
    :param value: pad value(int or tuple)
    :return: opencv image
    """
    channel = 1 if len(img.shape) == 2 else img.shape[2]
    if isinstance(value, int):
        value = tuple([value] * channel)
    N = max(img.shape[:2])
    dst = np.zeros((N, N, img.shape[2]), dtype=np.uint8) + np.array(value, dtype=np.uint8)
    dx = (N - img.shape[1]) // 2
    dy = (N - img.shape[0]) // 2
    dst[dy:dy + img.shape[0], dx:dx + img.shape[1]] = img
    return dst


def _to_image_list(args):
    imgs = []
    for arg in args:
        if isinstance(arg, list):
            imgs += arg
        elif isinstance(arg, np.ndarray):
            imgs.append(arg)
        else:
            pass
    return imgs


def hconcat(*args):
    """
    Return the input images horizontally.
    :param args: opencv image list OR comma seperated images
    :return: opencv image
    """
    imgs = _to_image_list(args)
    max_height = max(img.shape[0] for img in imgs)
    rimgs = [to_color(resize(img, height=max_height)) for img in imgs]
    return cv2.hconcat(rimgs)


def vconcat(*args):
    """
    Return the input images vertically.
    :param args: opencv image list OR comma seperated images
    :return: opencv image
    """
    imgs = _to_image_list(args)
    max_width = max(img.shape[1] for img in imgs)
    rimgs = [to_color(resize(img, width=max_width)) for img in imgs]
    return cv2.vconcat(rimgs)


def canny(img: np.ndarray):
    if len(img.shape) == 3:
        if img.shape[2] == 3:
            img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        elif img.shape[2] == 4:
            img = cv2.cvtColor(img, cv2.COLOR_BGRA2GRAY)

    high_th, _ = cv2.threshold(img, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
    low_th = high_th / 2
    return cv2.Canny(img, low_th, high_th)


if __name__ == '__main__':
    # sam, issm, uiq
    img1 = imread("../data/computer01.jpg")
    img2 = imread("../data/computer02.jpg")
    # r = _compare(img1, img2, rmse)
    hash_funcs = [average_hash, colorhash, phash, phash_simple, whash, dhash, dhash_vertical, crop_resistant_hash]
    for hash_func in hash_funcs:
        r = hash_func(to_pil(img1))
        print(r)



import pytz
import logging
import datetime


class Formatter(logging.Formatter):
    """override logging.Formatter to use an aware datetime object"""

    def converter(self, timestamp):
        # Create datetime in UTC
        dt = datetime.datetime.fromtimestamp(timestamp, tz=pytz.UTC)
        # Change datetime's timezone
        return dt.astimezone(pytz.timezone('Asia/Seoul'))

    def formatTime(self, record, datefmt=None):
        dt = self.converter(record.created)
        if datefmt:
            s = dt.strftime(datefmt)
        else:
            try:
                s = dt.isoformat(timespec='milliseconds')
            except TypeError:
                s = dt.isoformat()
        return s


console = logging.FileHandler('UpdateSubsStatus.log')
console.setFormatter(Formatter(
        '%(asctime)s;%(name)s - %(levelname)s - %(message)s', '%Y-%m-%d %H:%M:%S'))
logging.getLogger('').addHandler(console)
logging.warning("Critical event")

with open('UpdateSubsStatus.log', "rt", encoding="utf-8") as f:
    print(f.read())



from zeroset import cv0
import cv2
from zeroset import viz0

# img1 = cv0.from_url("https://ae01.alicdn.com/kf/H6717450a87eb42ec9733cc27d1c52d5aa.jpg")
#
# img2 = cv0.from_blank(500, 500, (24, 231, 124))
#
# img3 = viz0.write_fit_text(width=1000, height=400, text="안녕 디지몬", lr_pad=10, tb_pad=10)

# print(cv0.similarity.psnr(img1, img2))
#
# print(cv0.hash.average_hash(img1))
#
# cv0.exit()

# img = cv0.hconcat(img1, img2)

# cv0.imshow("img", img3)
# cv0.waitKey()

print(*dir(viz0.cv2), sep="\n")



# from zeroset import py0
# import numpy as np
# import sys
#
# a = np.zeros((300, 300, 3), dtype=np.uint8)
#
# print(py0.get_value_size(a))
#
# print(py0._format_file_size(sys.getsizeof(a)))

from zeroset import log0
import logging

logging.debug('This message is a log message.')
logging.info('This message is a log message.')
logging.warning('This message is a log message.')
logging.error('This message is a log message.')
logging.critical('This message is a log message.')



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
        name="zeroset",
        version="0.1.3",
        author="Bomm Kim",
        author_email="springnode@gmail.com",
        description="Useful collection of features.",
        long_description=long_description,
        long_description_content_type="text/markdown",
        url="https://github.com/springkim/zeroset",
        packages=setuptools.find_packages(),
        classifiers=[
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
        ],
        python_requires='>=3.6',
        install_requires=[
            'natsort>=8.0.0',
            'Pillow>=10.0.0',
            'opencv-python',
            'screeninfo>=0.8.0',
            'tabulate>=0.9.0',
            'pytz',
            'selenium>=4.0.0',
            'webdriver-manager>=4.0.0',
            'screeninfo',
            'gif',
            'chromedriver_autoinstaller',
            'image_similarity_measures',
            'pyfftw',
            'imagehash'
        ],
)



import math
import os
import sys
import pickle


def format_file_size(size_bytes: int, precision: int = 2, padding: str = ''):
    if size_bytes == 0:
        return "0B"
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size_bytes, 1024)))
    p = math.pow(1024, i)
    s = round(size_bytes / p, precision)
    return f'{s}{padding}{size_name[i]}'


def get_value_size(val):
    return format_file_size(sys.getsizeof(pickle.dumps(val)))


def get_file_size(file):
    return format_file_size(os.path.getsize(file))


def get_basename(file):
    return os.path.splitext(os.path.basename(file))[0]


def ms_to_hms(milliseconds):
    hour = milliseconds // 3600000
    milliseconds %= 3600000
    minute = milliseconds // 60000
    milliseconds %= 60000
    second = milliseconds // 1000
    milliseconds %= 1000
    return hour, minute, second, milliseconds


def s_to_hms(seconds):
    seconds = seconds % (24 * 3600)
    hour = seconds // 3600
    seconds %= 3600
    minutes = seconds // 60
    seconds %= 60
    return hour, minutes, seconds
